
#   MIDAS Server for the TapeServer service

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]      ;#  load default support procedures

global MIDAS_TapeServer_Verbose
set MIDAS_TapeServer_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

package require TapeServerCmd

# Use namespaces to isolate your methods
namespace eval urn:TapeServer {

    
    proc MountVolume {id volume_name} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method MountVolume with id = $id; volume name = $volume_name"
        return [rpcvar::rpcvar string "[TapeServer_MountVolume $id $volume_name]"]
    }

    proc OpenFile {id access_mode label_type record_length block_length file_name} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method OpenFile with id = $id; access_mode = $access_mode; label_type = $label_type;\
              record length = $record_length; block length = $block_length; file name = $file_name"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireDeviceStatus {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireDeviceStatus with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc CloseFile {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method CloseFile with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc DismountVolume {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method DismountVolume with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc DeallocateDevice {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method DeallocateDevice with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireStreamStatus {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireStreamStatus with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InitialiseVolume {id volume_name label_type density_code current_volume_name} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InitialiseVolume with id = $id; label type = $label_type; \
           density code = $density_code current_volume_name = $current_volume_name"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc IdentifyVolume {id} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method IdentifyVolume with id = $id"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireAvailableDevices {} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireAvailableDevices"

        return [rpcvar::rpcvar string "[TapeServer_InquireAvailableDevices]"]
    }

    proc AllocateDevice {id device_name} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method AllocateDevice with id = $id; device_name = $device_name"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc ClaimTapeServer {} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method ClaimTapeServer"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc FreeTapeServer {} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method FreeTapeServer"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc SetTapeServerState {state} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method SetTapeServerState with state = $state"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireTapeServerState {} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireTapeServerState"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc AssociateStream {stream ids} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method AssociateStream with stream = $stream; ids = $ids"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireStreamAssociation  {stream} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireStreamAssociation with stream = $stream"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireStreamState {stream} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireStreamState with stream = $stream"


        return [rpcvar::rpcvar string "0 OK"]
    }

    proc SetRunNumber {n} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method SetRunNumber with n = $n"

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InquireAcqStatus {} {
        global MIDAS_TapeServer_Verbose
        if (MIDAS_TapeServer_Verbose) puts "called method InquireAcqStatus"

        return [rpcvar::rpcvar string "0 OK"]
    }


    # We have to publish the public methods...

     SOAP::export MountVolume 
     SOAP::export OpenFile
     SOAP::export InquireDeviceStatus 
     SOAP::export CloseFile 
     SOAP::export DismountVolume 
     SOAP::export DeallocateDevice
     SOAP::export InquireStreamStatus 
     SOAP::export InitialiseVolume
     SOAP::export IdentifyVolume
     SOAP::export InquireAvailableDevices
     SOAP::export AllocateDevice
     SOAP::export ClaimTapeServer
     SOAP::export FreeTapeServer
     SOAP::export SetTapeServerState
     SOAP::export InquireTapeServerState 
     SOAP::export AssociateStream
     SOAP::export InquireStreamAssociation 
     SOAP::export InquireStreamState
     SOAP::export SetRunNumber
     SOAP::export InquireAcqStatus

}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /TapeServer -namespace urn:TapeServer -uri urn:TapeServer]
